#include <stdio.h>
#include <time.h>

#ifdef UNROLLED
#include "unrolled_linked_list.h"
#else
#include "linked_list.h"
#endif

#define PRINT_ELAPSED(pms, pstart, pstop)    do {\
                *(pms) = ((pstop)->tv_sec - (pstart)->tv_sec) * 1E3 +\
                     ((pstop)->tv_nsec - (pstart)->tv_nsec) / 1E6;\
                printf("Elapsed milliseconds: %f\n", *(pms));} while(0)

#define ELEMENTS    1<<16

int main(int argc, char* argv[]) {
    double ms;
    int i, j;
    struct timespec start, stop;

    #ifdef UNROLLED
    unrolled_list_t *list = new_unrolled_list();
    #else
    linked_list_t *list = new_linked_list();
    #endif

    /* Experiment 1 */
    printf("Creating list with %d elements...\n", ELEMENTS);
    clock_gettime(CLOCK_PROCESS_CPUTIME_ID, &start);
    for (i=0; i<ELEMENTS; ++i) {
        append_elem(list, i);
    }
    clock_gettime(CLOCK_PROCESS_CPUTIME_ID, &stop);
    PRINT_ELAPSED(&ms, &start, &stop);

    /* Experiment 2 */
    printf("Performing get_elem[i++] till the last valid index...\n");
    clock_gettime(CLOCK_PROCESS_CPUTIME_ID, &start);
    for (i=0; i<ELEMENTS; ++i) {
        int x = get_elem(list, i);
    }
    clock_gettime(CLOCK_PROCESS_CPUTIME_ID, &stop);
    PRINT_ELAPSED(&ms, &start, &stop);

    /* Experiment 3 */
    printf("Performing get_elem[i--] from the last index till 0...\n");
    clock_gettime(CLOCK_PROCESS_CPUTIME_ID, &start);
    for (i=ELEMENTS-1; i>=0; i--) {
        int x = get_elem(list, i);
    }
    clock_gettime(CLOCK_PROCESS_CPUTIME_ID, &stop);
    PRINT_ELAPSED(&ms, &start, &stop);

    return 0;
}
