#include <stdlib.h>
#include <stdio.h>
#include "linked_list.h"

static node_t* new_node() {
	node_t *n = malloc(sizeof(node_t));

	if (n == NULL) {
		printf("Allocation error\n");
		exit(1);
	}

	n->next = NULL;

	return n;
}

linked_list_t* new_linked_list() {
	linked_list_t *list = malloc(sizeof(linked_list_t));

	if (list == NULL) {
		printf("Invalid list!\n");
		exit(1);
	}

	list->head = NULL;
	return list;
}

void append_elem(linked_list_t *list, int x) {

	if (list == NULL) {
		printf("Invalid list!\n");
		exit(1);
	}

	if (list->head == NULL) {
		list->head = new_node();
        list->head->data = x;
	} else {
		node_t *tail = list->head;
		while (tail->next != NULL)
			tail = tail->next;

		tail->next = new_node();
        tail->next->data = x;
	}
}

int get_elem(linked_list_t *list, int i) {

	if (list == NULL) {
		printf("Invalid list!\n");
		exit(1);
	}

	node_t *n = list->head;
	while (n != NULL && i-- > 0) {
        n = n->next;
    }

    if (n) return n->data;

	printf("Invalid index!\n");
	exit(1);
}

void remove_elem(linked_list_t *list, int i) {

	if (list == NULL) {
		printf("Invalid list!\n");
		exit(1);
	}

	node_t *n = list->head;
	node_t *prev = NULL;
	while (n != NULL && i-- > 0) {
        prev = n;
        n = n->next;
    }

    if (n == NULL) {
        printf("Invalid index!\n");
        exit(1);
    }

    if (prev == NULL) {
        list->head = n->next;
    } else {
        prev->next = n->next;
    }

    free(n);
}

int list_size(linked_list_t *list) {

	int count = 0;

	node_t *n = list->head;
	while (n != NULL) {
		n = n->next;
        count++;
	}

	return count;
}

void delete_linked_list(linked_list_t *list) {

	node_t *n = list->head;
	while (n != NULL) {
		node_t *current = n;
		n = n->next;
		free(current);
	}

	free(list);
}
